Encoding: UTF-8
Package: RSiena
Type: Package
Title: Siena - Simulation Investigation for Empirical Network Analysis
Version: 1.6.6
Date: 2026-04-20
Authors@R: c(person("Tom A.B.", "Snijders", role = c("aut", "ctb"), comment = c(ORCID = "0000-0003-3157-4157")),
              person("Ruth M.", "Ripley", role = "aut"),
              person("Krists", "Boitmanis", role = c( "aut","ctb")),
              person("Christian", "Steglich", email = "c.e.g.steglich@rug.nl", role = c("cre", "aut"), comment = c(ORCID = "0000-0002-9097-0873")),
			     person("Johan", "Koskinen", role = c("ctb"), comment = c(ORCID = "0000-0002-6860-325X")),
              person("Nynke M.D.", "Niezink", role = c("aut","ctb"), comment = c(ORCID = "0000-0003-4199-4841")),
              person("Viviana", "Amati", role = c("aut","ctb"), comment = c(ORCID = "0000-0003-1190-1237")),
              person("Christoph", "Stadtfeld", role = "ctb", comment = c(ORCID = "0000-0002-2704-2134")),
              person("James", "Hollway", role = "ctb", comment = c("IHEID", ORCID = "0000-0002-8361-9647")),
              person("Per", "Block", role = "ctb", comment = c(ORCID = "0000-0002-7583-2392")),
              person("Robert", "Krause", role = "ctb", comment = c(ORCID = "0000-0003-4288-4732")),
              person("Charlotte", "Greenan", role = "ctb"),
              person("Josh", "Lospinoso", role = "ctb"),
              person("Michael", "Schweinberger", role = "ctb", comment = c(ORCID = "0000-0003-3649-5386")),
              person("Mark", "Huisman", role = "ctb",comment = c(ORCID = "0000-0002-9009-7859")),
              person("Felix", "Schoenenberger", role = c("aut","ctb")),
              person("Mark", "Ortmann", role = "ctb"),
              person("Marion", "Hoffman", role = "ctb", comment=c(ORCID = "0000-0002-0741-7760")),
              person("Robert", "Hellpap", role = "ctb"),
              person("Alvaro", "Uzaheta", role = "ctb", comment=c(ORCID = "0000-0003-4367-3670")),
              person("Steffen", "Triebel", role = "ctb"),
              person("Daniel", "Gotthardt", role = "ctb", comment=c(ORCID = "0000-0002-1294-9913")))
Maintainer: Christian Steglich <c.e.g.steglich@rug.nl>
Depends: R (>= 4.5.0)
Imports: Matrix, lattice, parallel, MASS, methods, xtable, network
Suggests: tools, codetools, tcltk
SystemRequirements: GNU make
Description: The main purpose of this package is to perform simulation-based
   estimation of stochastic actor-oriented models for longitudinal network
   data collected as panel data. Dependent variables can be single or
   multivariate networks, which can be directed, non-directed, or two-mode;
   and associated actor variables.
   There are also functions for testing parameters and checking goodness of fit.
   An overview of these models is given in Snijders (2017),
   <doi:10.1146/annurev-statistics-060116-054035>.
License: GPL-2 | GPL-3 | file LICENSE
LazyData: yes
Biarch: yes
NeedsCompilation: yes
BuildResaveData: no
URL: https://www.stats.ox.ac.uk/~snijders/siena/
BugReports: https://github.com/stocnet/rsiena/issues
Packaged: 2026-04-20 08:00:41 UTC; cegst
Author: Tom A.B. Snijders [aut, ctb] (ORCID:
    <https://orcid.org/0000-0003-3157-4157>),
  Ruth M. Ripley [aut],
  Krists Boitmanis [aut, ctb],
  Christian Steglich [cre, aut] (ORCID:
    <https://orcid.org/0000-0002-9097-0873>),
  Johan Koskinen [ctb] (ORCID: <https://orcid.org/0000-0002-6860-325X>),
  Nynke M.D. Niezink [aut, ctb] (ORCID:
    <https://orcid.org/0000-0003-4199-4841>),
  Viviana Amati [aut, ctb] (ORCID:
    <https://orcid.org/0000-0003-1190-1237>),
  Christoph Stadtfeld [ctb] (ORCID:
    <https://orcid.org/0000-0002-2704-2134>),
  James Hollway [ctb] (IHEID, ORCID:
    <https://orcid.org/0000-0002-8361-9647>),
  Per Block [ctb] (ORCID: <https://orcid.org/0000-0002-7583-2392>),
  Robert Krause [ctb] (ORCID: <https://orcid.org/0000-0003-4288-4732>),
  Charlotte Greenan [ctb],
  Josh Lospinoso [ctb],
  Michael Schweinberger [ctb] (ORCID:
    <https://orcid.org/0000-0003-3649-5386>),
  Mark Huisman [ctb] (ORCID: <https://orcid.org/0000-0002-9009-7859>),
  Felix Schoenenberger [aut, ctb],
  Mark Ortmann [ctb],
  Marion Hoffman [ctb] (ORCID: <https://orcid.org/0000-0002-0741-7760>),
  Robert Hellpap [ctb],
  Alvaro Uzaheta [ctb] (ORCID: <https://orcid.org/0000-0003-4367-3670>),
  Steffen Triebel [ctb],
  Daniel Gotthardt [ctb] (ORCID: <https://orcid.org/0000-0002-1294-9913>)
Repository: CRAN
Date/Publication: 2026-04-20 11:20:02 UTC
Built: R 4.6.0; aarch64-apple-darwin23; 2026-04-26 09:06:26 UTC; unix
Archs: RSiena.so.dSYM
