useDynLib(RSiena, .registration = TRUE, .fixes="C_")

export(
	coCovar, coDyadCovar, varCovar, varDyadCovar, as_covariate_rsiena,
	sienaNet, sienaDependent, as_dependent_rsiena,
	getEffects, print01Report, write_report, siena07, siena,
	sienaCompositionChange, updateTheta, updateSpecification,
	sienaCompositionChangeFromFile, sienaDataCreate, make_data_rsiena,
	sienaGroupCreate, make_group_rsiena,
	sienaModelCreate, sienaAlgorithmCreate, 
	set_model_saom, set_algorithm_saom, set_output_saom, 
	sienaNodeSet, as_nodeset_rsiena, xtable.sienaFit, setEffect,
	includeEffects, includeInteraction, includeGMoMStatistics, 
   effectsDocumentation, sienaDataConstraint, print.xtable.sienaFit, 
	siena08, meta_siena, iwlsm, sienaTimeTest,
	includeTimeDummy, sienaGOF, descriptives, interpret_size,
	interpret_size_dynamics, sparseMatrixExtraction, networkExtraction, behaviorExtraction,
	OutdegreeDistribution, IndegreeDistribution, BehaviorDistribution,
	TriadCensus, mixedTriadCensus, dyadicCov, egoAlterCombi, egoAlterCovarComb,
	siena_table, xtable, score.Test, Wald.RSiena, Multipar.RSiena, 
   testSame.RSiena, funnelPlot, meta.table,
	influenceTable, selectionTable,
    as_composition_rsiena,as_composition_file_rsiena, make_constraint, 
	make_specification, set_effect, set_interaction, 
   update_theta, update_specification, 
	test_parameter, write_result, test_time, test_gof,
	interpret_selection, interpret_influence, transformScript)
importFrom("Matrix", "Matrix", "sparseMatrix", "spMatrix", "drop0", 
           "rankMatrix", "mat2triplet")
importFrom("lattice", "xyplot", "trellis.par.get", "trellis.par.set", "bwplot",
           "panel.violin", "panel.bwplot", "panel.xyplot", "panel.text",
           "panel.abline") 
importFrom("parallel", "clusterCall", "parLapply", "parRapply",
           "clusterSetRNGStream", "makeCluster", "stopCluster")
importFrom("xtable", "xtable", "print.xtable")
importFrom("MASS", "cov.rob", "ginv")
importFrom("grDevices", "rgb", "xy.coords")
importFrom("graphics", "axis", "barplot", "layout", "lines",
           "mtext", "pairs", "panel.smooth", "par", "pie", "plot",
           "points", "strwidth", "text")
importFrom("methods", "as", "is", "hasArg")
importFrom("stats", ".getXlevels", "acf", "as.formula", "coef",
           "contr.helmert", "cor", "cor.test", "cov", "density",
           "dnorm", "ecdf", "hatvalues", "lm", "lm.wfit", "mad",
           "mahalanobis", "median", "model.matrix", "model.offset",
           "model.response", "model.weights", "na.omit", "naprint",
           "optim", "optimize", "pchisq", "plot.ts", "pnorm", 
           "predict.lm", "pt", "qchisq", "qnorm", "quantile",
           "rWishart", "rnorm", "runif", "sd", "setNames", "ts", "uniroot", 
           "var", "weighted.mean", "weights")
importFrom("utils", "browseURL", "edit",
           "flush.console", "getFromNamespace", "object.size",
           "packageDescription", "read.csv",
           "read.table", "write.csv", "write.table")
importFrom("network", "network.initialize", "network.edgelist")
S3method(print, sienadata)
S3method(print, siena)
S3method(make_specification, sienadata)
S3method(make_specification, siena)
S3method(make_specification, sienaGroup)
S3method(make_constraint, sienadata)
S3method(make_constraint, siena)
S3method(make_constraint, sienaGroup)
S3method(write_report, sienadata)
S3method(print, sienaGroup)
S3method(print, sienaDependent)
S3method(print, sienaFit)
S3method(summary, sienaFit)
S3method(coef, sienaFit)
S3method(vcov, sienaFit)
S3method(test_parameter, sienaFit)
S3method(write_result, sienaFit)
S3method(print, summary.sienaFit)
S3method(write_result, sienaFit)
S3method(test_time, sienaFit)
S3method(test_gof, sienaFit)
S3method(interpret_selection, sienaFit)
S3method(interpret_influence, sienaFit)
S3method(interpret_size, sienaFit)
S3method(print, sienaAlgorithm)
S3method(print, sienaMeta)
S3method(print, summary.sienaMeta)
S3method(summary, sienaMeta)
S3method(plot, sienaMeta)
S3method(addAttributes, coCovar)
S3method(addAttributes, varCovar)
S3method(addAttributes, coDyadCovar)
S3method(addAttributes, varDyadCovar)
S3method(print, sienaTest)
S3method(print, sienaTimeTest)
S3method(summary, sienaTimeTest)
S3method(print, summary.sienaTimeTest)
S3method(plot, sienaTimeTest)
S3method(print, sienaEffects)
S3method(summary, sienaEffects)
S3method(set_effect, sienaEffects)
S3method(set_interaction, sienaEffects)
S3method(print, summary.sienaEffects)
S3method(edit, sienaEffects)
S3method(interpret_size, sienaEffects)
S3method(update_theta, sienaEffects)
S3method(update_specification, sienaEffects)
S3method(print, xtable.sienaFit)
S3method(summary, sienaGOF)
S3method(print, sienaGOF)
S3method(plot, sienaGOF)
S3method(descriptives, sienaGOF)
S3method(summary, sienaRI)
S3method(print, sienaRI)
S3method(plot, sienaRI)
S3method(summary, sienaRIDynamics)
S3method(print, sienaRIDynamics)
S3method(plot, sienaRIDynamics)
S3method(predict, iwlsm)
S3method(print, iwlsm)
S3method(print, summary.iwlsm)
S3method(summary, iwlsm)
S3method(iwlsm, formula)
S3method(iwlsm, default)
S3method(se.contrast, iwlsm)
S3method(vcov, iwlsm)
S3method(print, chains.data.frame)
S3method(print, influenceTable)
S3method(print, selectionTable)
